@@ -17,7 +17,7 @@
 """Python bindings for Subversion."""
 
 __author__ = "Jelmer Vernooij <jelmer@samba.org>"
-__version__ = (0, 8, 3)
+__version__ = (0, 8, 7)
 
 NODE_DIR = 2
 NODE_FILE = 1
@@ -30,6 +30,7 @@ ERR_RA_SVN_CONNECTION_CLOSED = 210002
 ERR_WC_LOCKED = 155004
 ERR_RA_NOT_AUTHORIZED = 170001
 ERR_INCOMPLETE_DATA = 200003
+ERR_DIR_NOT_EMPTY = 200011
 ERR_RA_SVN_MALFORMED_DATA = 210004
 ERR_RA_NOT_IMPLEMENTED = 170003
 ERR_FS_NO_SUCH_REVISION = 160006
@@ -72,6 +73,7 @@ ERR_NODE_UNKNOWN_KIND = 145000
 ERR_RA_SERF_SSL_CERT_UNTRUSTED = 230001
 ERR_ENTRY_NOT_FOUND = 150000
 ERR_BAD_PROPERTY_VALUE = 125005
+ERR_FS_ROOT_DIR = 160021
 
 ERR_APR_OS_START_EAIERR = 670000
 ERR_APR_OS_ERRSPACE_SIZE = 50000
