@@ -13,8 +13,9 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+"""Tests for subvertpy.delta."""
+
 from cStringIO import StringIO
-from unittest import TestCase
 
 from subvertpy.delta import (
     decode_length,
@@ -23,6 +24,7 @@ from subvertpy.delta import (
     send_stream,
     unpack_svndiff0,
     )
+from subvertpy.tests import TestCase
 
 class DeltaTests(TestCase):
 
