@@ -26,7 +26,6 @@
 #include <apr_errno.h>
 #include <svn_error_codes.h>
 #include <svn_config.h>
-#include <svn_version.h>
 #include <svn_path.h>
 #include <svn_props.h>
 
@@ -705,6 +704,8 @@ static void stream_dealloc(PyObject *self)
 	StreamObject *streamself = (StreamObject *)self;
 
 	apr_pool_destroy(streamself->pool);
+
+	PyObject_Del(self);
 }
 
 static PyObject *stream_init(PyTypeObject *type, PyObject *args, PyObject *kwargs)
